#[===================================================================[
   multiconfig misc
#]===================================================================]

if (is_multiconfig)
  # This code finds all source files in the src subdirectory for inclusion
  # in the IDE file tree as non-compiled sources. Since this file list will
  # have some overlap with files we have already added to our targets to
  # be compiled, we explicitly remove any of these target source files from
  # this list.
  file (GLOB_RECURSE all_sources RELATIVE ${CMAKE_CURRENT_SOURCE_DIR}
    CONFIGURE_DEPENDS
    src/*.* Builds/*.md docs/*.md src/*.md Builds/*.cmake)
  file(GLOB md_files RELATIVE ${CMAKE_CURRENT_SOURCE_DIR} CONFIGURE_DEPENDS
    *.md)
  LIST(APPEND all_sources ${md_files})
  foreach (_target secp256k1::secp256k1 ed25519::ed25519 xrpl_core xrpl_plugin rippled)
    get_target_property (_type ${_target} TYPE)
    if(_type STREQUAL "INTERFACE_LIBRARY")
      continue()
    endif()
    get_target_property (_src ${_target} SOURCES)
    list (REMOVE_ITEM all_sources ${_src})
  endforeach ()
  if (tests)
    foreach (_target plugin_test_setregularkey
              plugin_test_trustset
              plugin_test_escrowcreate
              plugin_test_badtransactor
              plugin_test_badledgerentry
              plugin_test_badstypeid
              plugin_test_badsfieldtypeid
              plugin_test_badsfieldtypepair
              plugin_test_badtercode
              plugin_test_badinnerobject)
      get_target_property (_type ${_target} TYPE)
      if(_type STREQUAL "INTERFACE_LIBRARY")
        continue()
      endif()
      get_target_property (_src ${_target} SOURCES)
      list (REMOVE_ITEM all_sources ${_src})
    endforeach ()
  endif ()
  target_sources (rippled PRIVATE ${all_sources})
  set_property (
    SOURCE ${all_sources}
    APPEND
    PROPERTY HEADER_FILE_ONLY true)
  if (MSVC)
    set_property(
      DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
      PROPERTY VS_STARTUP_PROJECT rippled)
  endif ()

  group_sources(src)
  group_sources(docs)
  group_sources(Builds)
endif ()
